#include "Service.h"

/******************************************************
 * Initialize with TCP/IP service-name or port and protocol
 * PRE: name is a valid TCP/IP service or a port number.
 * POST: isCopy == false and myProtocol == protocol &&
 *        myName == the name of the service (or 'user-defined') &&
 *        myPort == the (network byte order) port number.
 ********************************************************/

void Service::initializeWith(const string & name)
{
  initProtocol();  // polymorphic, defined by subclasses

  // first see if name is the name of a valid TCP/IP service
  struct servent * ptr;
  if ((ptr = getservbyname(name.c_str(),
			  myProtocolInfo->name().c_str())))
    {
      myName = name;
      myPort = ptr->s_port;
    }
  // if not, see if it is the port of a valid TCP/IP service
  else if ((ptr = getservbyport(htons((unsigned short)atoi(name.c_str())),
			       myProtocolInfo->name().c_str())))
    {
      myName = ptr->s_name;
      myPort = ptr->s_port;
    }
  // if not, see if it is a nonstandard TCP/IP port
  else if ((myPort = htons((unsigned short) atoi(name.c_str()))) == 0)
    {
      myName = "user-defined";
    }
  // if not, we've exhausted our options...
  else
    fatal("Service: Unable to find port for %s", name.c_str());
}

/******************************************************
 * Initialize with TCP/IP port.
 * PRE: port is the number of a TCP/IP (or unassigned) port.
 * POST: isCopy == false and myProtocol == protocol &&
 *        myName == the name of the service (or 'user-defined') &&
 *        myPort == the (network byte order) port number.
 ********************************************************/

void Service::initializeWith(const unsigned short port)
{
  initProtocol();       // polymorphic, defined in subclasses

  myPort = htons(port);
  struct servent * ptr;
  if ((ptr = getservbyport(myPort,
			   myProtocolInfo->name().c_str())))
    myName = ptr->s_name;
  else 
    myName = "user-defined";
}

/******************************************************
 * (Polymorphic) print function (used by <<)
 * POST: RV == my name, port, and protocol.
 ******************************************************/
void Service::print(ostream & out) const
{
  out << myProtocolInfo->name() << ' '
      << name() << " service, port: "
      << portHBO();
}


/******************************************************
 * (Polymorphic) compare() (used by relational operators)
 * PRE: netObj is a Service using the same protocol.
 * POST: RV == 0 iff we are using the same port and protocol,
 *            -X iff myPort < netObj.Port
 *            +X iff myPort > netObj.Port().
 ******************************************************/

int Service::compare(const NetworkObject & netObj) const
{
  const Service * ptr = dynamic_cast<const Service*>(&netObj);

  if (!ptr)
    fatal("Service::compare(): Unable to compare %s and %s",
	  className().c_str(), netObj.className().c_str());

  if (*(protocolInfo()) == *(ptr->protocolInfo()))
    return portHBO() - ptr->portHBO();
  else
    return -1;  // false, due to different protocols
}


/*****************************************************************
 * Copy Constructor
 * PRE: s is an object which inherits from Service
 * POST: data members are equal to s's data memebers.
 *****************************************************************/
inline Service::Service(const Service & s)
{
   this->myName = s.myName;
   this->myPort = s.myPort;
   this->myProtocolInfo = s.myProtocolInfo;
}

/*****************************************************************
 * operator= - implements the assignment operator
 * PRE: s is an object which inherits from Service
 * POST: data members are equal to s's data memebers.
 *****************************************************************/
inline Service &Service::operator=(const Service & s)
{
   this->myName = s.myName;
   this->myPort = s.myPort;
   this->myProtocolInfo = s.myProtocolInfo;
   return *this;
}

