/* Communicator.h defines an abstract base class for client and server
 *  sockets.
 *
 * Author: Joel C. Adams
 * Date: October, 1998.
 * Version 0.9
 *
 * Modified: Ryan VanderBijl
 * Date:     May/June, 2000
 *
 * TO DO:   add send, receive for fundamental types...
 */

#ifndef COMMUNICATOR
#define COMMUNICATOR

#include "NetworkObject.h"
#include "Service.h"
#include "Host.h"
#include "NetworkAddress.h"
using namespace std;

class Communicator : public NetworkObject
{
 public:
  Communicator();
  Communicator(const Communicator & comm);
  virtual ~Communicator();

  void start(Service * servicePtr);
  bool isOn() const;
  void stop();

  virtual int send(const string & msg) = 0;
  virtual int receive(string & msg) = 0;

  // add send, receive for fundamental types...

  Service  * serviceInfo () const;
  Protocol * protocolInfo() const;

  virtual string className() const;
  virtual void print(ostream & out) const;
  virtual int compare(const NetworkObject & socket) const;
  Communicator & operator=(const Communicator & comm);

 protected:
  void setServiceInfo(Service * service);
  virtual void allocate() = 0;

 private:
  Service *           myServiceInfo;
  bool                myIsOn;
};




/*****************************************************
 * Copy Constructor
 * PRE: comm is an object inherited from Communicator
 * POST: Data members have been set to be equal to comm's
 ****************************************************/
inline Communicator::Communicator(const Communicator & comm)
{
    this->myIsOn = comm.myIsOn;
    this->myServiceInfo = comm.myServiceInfo;
}

/*****************************************************
 * operator= -- implements the a = b operator
 * PRE: comm is an object inherited from Communicator
 * POST: Data members have been set to be equal to comm's
 ****************************************************/
inline Communicator & Communicator::operator=(const Communicator & comm)
{
    this->myIsOn = comm.myIsOn;
    this->myServiceInfo = comm.myServiceInfo;
    return *this;
}

/*********************************
 * Check whether Communicator is on (being used).
 * POST: RV == isOn.
 *********************************/
inline bool Communicator::isOn() const
{
  return myIsOn;
}

/***************************************************
 * Default constructor
 * POST: data members have been initialized with default values.
 ***************************************************/

inline Communicator::Communicator()
{
  myIsOn = false;
  myServiceInfo = NULL;
}

/*****************************************************
 * stop Communicator (Called by subclasses).
 * POST: isOn == false.
 *****************************************************/
inline void Communicator::stop()
{
  delete myServiceInfo; 
  myIsOn = false;       
}

/******************************************************
 * Destructor.
 * POST: Dynamically allocated memory has been reclaimed
 ******************************************************/
inline Communicator::~Communicator()
{
  if (myIsOn)
    stop();
}

/******************************************************
 * Protocol information accessor (via Service)
 * POST: RV == my protocol information.
 ******************************************************/

inline Protocol * Communicator::protocolInfo() const
{
  return myServiceInfo->protocolInfo();
}


/******************************************************
 * Service accessor.
 * POST: RV == myService.
 ******************************************************/
inline Service * Communicator::serviceInfo() const
{
   return myServiceInfo;
}


/******************************************************
 * (Polymorphic) Name of class (used by diagnostics).
 * POST: RV == "Communicator".
 ******************************************************/

inline string Communicator::className() const
{
   return "Communicator";
}

/*****************************************************
 * (Polymorphic) print() (called by operator<<)
 * PRE: out is an open ostream.
 * POST: my information has been inserted into out.
 *****************************************************/

inline void Communicator::print(ostream & out) const
{
  if (myServiceInfo) 
      out << "Communicator for " << (*myServiceInfo);
  else 
      out << "Communicator for no one.";
}
#endif
