/***************************************************
 * PoolManger.h 
 *
 * Defines a class, PoolManager, which holds a list
 * of PoolItems (well, actually, just a list of items, which ought
 * to inherit from PoolItem, or at least implement a isBusy()).
 * It really holds a list of class T ( == items ;-).
 *
 * This class allows you to specify how many items to start with,
 * and the maximum number of items. The basic operation is 
 * getItem, which returns a pointer to an item. This operation
 * goes through the list which is maintained in order of 
 * least-recently-used first. It returns the first non-busy item
 * (ie the call to isBusy()). If all items are busy, and we have not
 * counted upto the maximum number of items, we will create a new item.
 *
 */

#ifndef POOL_MANAGER
#define POOL_MANAGER
using namespace std;

// Visual C++ 6.0 dies if these are const static in the class.
#define INITIAL_NUM_ITEMS  1 
#define MAX_NUM_ITEMS      15

template<class T>
class PoolManager {
  public:
     PoolManager();
     PoolManager(int init, int max);
     virtual ~PoolManager();

     virtual T* getItem();

  private:
     void fatal(char *);
     void init_items(int n);

     int myMax, myCount;

     typedef struct _list_node {
	T * data;
	struct _list_node *next;
     } my_list_node;

     my_list_node *myHead, *myTail;
};


/*************
 * Constructor
 *  make a pool manager, with an init number of items, and a maximum
 *  of max items.
 *  One item is always created, even if init == 0. 
 * PRE: init <= max.  
 * POST: init items have been created, and are ready to be used.
 *************/
template<class T>
inline PoolManager<T>::PoolManager(const int init, const int max) {
    if (init > max) {
       fatal("Can't start with more items that the maximum number!");
    }
    myMax = max;
    init_items(init);
}

/*************************
 * Default Constructor
 *   Create a poolmanger, with a default number of items(5), and 
 *   a default maximum(15).
 *************************/
template<class T>
inline PoolManager<T>::PoolManager() {
    myMax = MAX_NUM_ITEMS;
    init_items(INITIAL_NUM_ITEMS);
}


/*********************
 * Deconstructor
 *  destorys the list, and deletes all items.
 ********************/
template<class T>
inline PoolManager<T>::~PoolManager() 
{
   my_list_node *it = myHead, *last = NULL;
   while(it) {
      delete it->data;
      last = it;
      it = it->next;
      delete last;
   }
}

/**********
 * fatal - create a fatal error message.
 **********/
template<class T>
inline void PoolManager<T>::fatal(char * s) {
    cerr << "PoolManager: " << s << endl;
    exit;
}

/******************************
 * getItem -- the grunt work
 *   This function looks through the list of items for a NON-busy
 *   item. If it finds a non-busy item, it moves the item to the
 *   back of the list, and returns a pointer to it.
 *   If no NON-busy items can be found, AND the maximum number of
 *   items has NOT been reached, then a new item is created, put at
 *   the end of the list, and returned. If the max has been reached,
 *   NULL is returned.
 *   This list is kept in a least-recently-used-first order. This
 *   is because items may become not-busy in any order, especially
 *   in a threaded envrionment which this was created for.
 * PRE: PoolManager exists
 * POST: RV == pointer to an non-busy item, or == NULL, if one may
 *       not be created.
 ******************************/
template<class T>
T* PoolManager<T>::getItem() {
    my_list_node *it = myHead, *prev = NULL;

    while(it != NULL) {
	if (! it->data->isBusy() ) {
	    // The item isn't busy, so we want to remove it from
	    //   where it is currently in the list, and put it
	    //   at the end of the list.
	    if (it == myHead) { // Grabbing the first item
		myHead = it->next;
		if (!myHead) myHead = it; // For a one item pool.
		else {
		   it->next = NULL;
		   myTail->next = it;
		   myTail = myTail->next;
		}
	    } else if (it == myTail) { // Grabbing the last item.
		// we dont need to change the list
	    } else {  // Grabbing somewhere in the middle
	        prev->next = it->next;
		it->next = NULL;
		myTail->next = it;
		myTail = myTail->next;
	    }
	    return it->data;
	}
	prev = it;
	it = it->next;
    }

    if (myCount < myMax) {
	myCount++;
	it = new my_list_node;
	it->next = NULL;
	it->data = new T();
	myTail->next = it;
	myTail = myTail->next;
	return it->data;
    } 
    cerr << "PoolManager::getItem: not allowed to create a "
	 << "new item because max count (" << myMax 
	 << ") has been reached. Avoiding busy waiting: " 
	 << "returning NULL.\n" << flush;
    return NULL;
}


/****************
 * init_items
 *  -creates the initial pool of items.
 *  -always creates at least one item.
 ****************/
template<class T>
void PoolManager<T>::init_items(const int n)
{
    myTail = myHead = new my_list_node;
    myHead->next = NULL;
    myHead->data = new T();
    myCount = 1;

    for(int i = 1; i < n; ++i) {
	myTail->next = new my_list_node;
	myTail = myTail->next;
	myTail->data = new T();
	myCount++;
    }
}




#endif /*POOL_MANAGER*/
