/* Service.h defines an abstraction for TCP/IP services.
 *
 * Author: Joel C. Adams.
 * Date: October, 1998
 * Version 0.8
 */

#ifndef SERVICE
#define SERVICE

#include "Proto.h"        // Protocol
#include "NetworkObject.h"   // base class
using namespace std;

class Service : public NetworkObject
{
 public:
  Service(const string & name);
  Service(const unsigned short port);
  Service(const string & name, const unsigned short port);
  Service(const unsigned short port, const string & name);
  Service(const Service & );

  virtual ~Service();

  string name() const;
  unsigned short portHBO() const;
  unsigned short portNBO() const;
  Protocol * protocolInfo() const;

  virtual void print(ostream & out) const;
  virtual int compare(const NetworkObject & netObj) const;
  virtual string className() const;
  Service &operator=(const Service & s);

  virtual int BSD_Info() const = 0;
  virtual char * TLI_Info() const = 0;
  
 protected:
  Service();
  void initializeWith(const string & name);
  void initializeWith(const unsigned short port);
  void initializeWith(const string & name, const unsigned short port);
  virtual void initProtocol() = 0;
  void setProtocolInfo(Protocol * protocol);

 private:
  string               myName;
  unsigned short       myPort;
  Protocol *           myProtocolInfo;
};

/******************************************************
 * Default constructor
 * POST: my data members have received default values.
 ******************************************************/

inline Service::Service()
{
  myPort = 0;
  myProtocolInfo = NULL;
}



/******************************************************
 * Construct with TCP/IP service-name or port and protocol
 * PRE: name is a valid TCP/IP service or a port number.
 *      protocol is a TCP or UDP Protocol.
 * POST: isCopy == false and myProtocol == protocol &&
 *        myName == the name of the service (or 'user-defined') &&
 *        myPort == the (network byte order) port number.
 ********************************************************/

inline Service::Service(const string & name)
{
  initializeWith(name);
}


/******************************************************
 * Construct with TCP/IP port
 * PRE: port is the number of a TCP/IP (or unassigned) port.
 * POST: isCopy == false and myProtocol == protocol &&
 *        myName == the name of the service (or 'user-defined') &&
 *        myPort == the (network byte order) port number.
 ********************************************************/

inline Service::Service(const unsigned short port)
{
  initializeWith(port);
}


/******************************************************
 * Initialize with name and TCP/IP port.
 *  (for creating a new service).
 * PRE: name is the name of the new service,
 *      port is the number of an unassigned port, and
 * POST: isCopy == false and myProtocol == protocol &&
 *        myName == the name of the service &&
 *        myPort == the (network byte order) port number.
 ********************************************************/

inline void Service::initializeWith(const string & name,
				    const unsigned short port)
{
  initProtocol();        // polymorphic, defined in subclasses

  myName = name;
  myPort = htons(port);
}

/******************************************************
 * Construct with name and TCP/IP port
 *  (for creating a new service).
 * PRE: name is the name of the service,
 *      port is the number of a port, and
 * POST: isCopy == false and myProtocol == protocol &&
 *        myName == the name of the service &&
 *        myPort == the (network byte order) port number.
 ********************************************************/

inline Service::Service(const string & name, const unsigned short port)
{
  initializeWith(name, port);
}

/******************************************************
 * Construct with TCP/IP port and name
 *  (for creating a new service).
 * PRE: name is the name of the service,
 *      port is the number of a port, and
 * POST: isCopy == false and myProtocol == protocol &&
 *        myName == the name of the service &&
 *        myPort == the (network byte order) port number.
 ********************************************************/

inline Service::Service(const unsigned short port, const string & name)
{
  initializeWith(name, port);
}


/******************************************************
 * Name accessor.
 * POST: RV == myName.
 ******************************************************/

inline string Service::name() const
{
  return myName;
}

/******************************************************
 * Port accessor (host byte-order).
 * POST: RV == myPort.
 ******************************************************/

inline unsigned short Service::portHBO() const
{
  return ntohs(myPort);
}


/******************************************************
 * Port accessor (network byte-order).
 * POST: RV == myPort.
 ******************************************************/

inline unsigned short Service::portNBO() const
{
  return myPort;
}


/******************************************************
 * Protocol accessor.
 * POST: RV == myProtocol.
 ******************************************************/

inline Protocol * Service::protocolInfo() const
{
  return myProtocolInfo;
}

/******************************************************
 * Protocol mutator.
 * PRE: protocol points to my Protocol information.
 * POST: myProtocolInfo = the information of protocol.
 ******************************************************/

inline void Service::setProtocolInfo(Protocol * protocol)
{
  myProtocolInfo = protocol;
}

inline string Service::className() const
{
  return "Service";
}


/******************************************************
 * Destructor
 * POST: If I am a copy, the space the copy constructor
 *       allocated for my servent has been deallocated.
 ******************************************************/

inline Service::~Service()
{
  // reclaim storage allocated by initProtocol() in subclasses
  delete myProtocolInfo;
}


#endif

