/********************************************************
 * ServiceThread.h 
 * 
 * Author: Ryan VanderBijl
 * Date:   June 2000
 *
 * This class is the base class for all C++ threads in cliser.
 * It inheritts the thread capabilities from PosixThread.h, a 
 * communicator stuff from CommunicatorUser.h. 
 * This class defines two need functions: interactWithClient, which 
 * must be defined by a child class, and setCommunicator, which allows
 * the parent thread to tell us how to communicate with a client.
 *
 */

#ifndef SERVICE_THREAD
#define SERVICE_THREAD

#include "PosixThread.h"
#include "CommunicatorUser.h"
using namespace std;

template<class T>
class ServiceThread : public CommunicatorUser<T>, public PosixThread
{
public:
    ServiceThread();
    virtual ~ServiceThread();
    virtual void run();
    virtual void interactWithClient() = 0;

    virtual void setCommunicator(T *communicator);
private:
};


/* Deconstructor -- does nothing */
template<class T>
inline ServiceThread<T>::~ServiceThread()
{ }

/***************************************************
 * run() -- calls interactWithClient
 *    called by PosixThread, when it is given the
 *    start messsage (er, when the main calls the
 *    start() function).
 ***************************************************/
template<class T>
inline void ServiceThread<T>::run()
{ 
   // optional stuff here 
   interactWithClient();

   // The thread needs to close the connection it is handling:
   communicatorInfo()->closeConnection();

   // optional stuff here
}


/***************************************************
 * Constructor  
 *   creates a generic communicator. This communicator
 *   will receive the details laster on from the parent
 *   thread.
 ***************************************************/
template<class T>
inline ServiceThread<T>::ServiceThread() : CommunicatorUser<T>(), PosixThread()
{
   T *comm = new T();
   CommunicatorUser<T>::setCommunicator(comm);
}

/***************************************************
 * setCommunicator
 *   sets the internal communicator equal to the
 *   new one.  This overwrites the CommunicatorUser's
 *   setCommunicator because we need a copy of the
 *   communicator, not a pointer to the same one.
 *   Otherwise, the main thread will modify the
 *   communicator and cause many many problems.
 * PRE: communicator is a communicator with a client
 * POST: our communicator has the same information
 *       so this tread can talk to the client.
 ***************************************************/
template<class T>
inline void ServiceThread<T>::setCommunicator(T *communicator) 
{
  *(communicatorInfo()) = *communicator;
}

#endif
