/* TLI_Communicator.h extends the Communicator abstraction with functionality
 *   specific to TLI sockets.
 *
 * Author: Joel Adams.
 * Date: November, 1998.
 * Version: 0.9
 *
 * Modified: Ryan VanderBijl
 * Date:     May/June, 2000
 *
 */

#ifndef TLI_COMMUNICATOR
#define TLI_COMMUNICATOR

#include <netinet/in.h>       // ntohl(), ntohs(), htons(), htonl()
#include "Communicator.h"
#include "SocketAddress.h"
#include <unistd.h>
#include <tiuser.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/fcntl.h>
#include <netinet/in.h>
#include <netdb.h>
using namespace std;


class TLI_Communicator : public Communicator
{
 public:
  TLI_Communicator();
  TLI_Communicator(const TLI_Communicator & comm);
  virtual ~TLI_Communicator();

  void start(Service * service);
  virtual void stop();

  virtual int receive(string & msg) = 0;
  virtual int send(const string & msg) = 0;

  SocketAddress * addressInfo() const;

  virtual int compare(const NetworkObject & netObj) const;
  virtual void print(ostream & out) const;
  virtual string className() const;
  TLI_Communicator & operator=(const TLI_Communicator & comm);

 protected:
  virtual int getDescriptor() const;

  virtual void allocate();
  void setAddressInfo(SocketAddress * address);

  virtual void fatal(const char * format, ...) const;
  virtual void warning(const char * format, ...) const;

 private:
  int myDescriptor;             // endpoint for TLI sockets
  SocketAddress * myAddressInfo;
};


/********************************************************
 * Default constructor.
 * POST: all data members have been initialized
 *        to default values.
 ********************************************************/

inline TLI_Communicator::TLI_Communicator()
  : Communicator()
{
  myDescriptor = -1;
  myAddressInfo = NULL;
}


/********************************************************
 * Open for a given service.
 * PRE: service points to a properly-defined Service.
 * POST: myServiceInfo points to service.
 ********************************************************/

inline void TLI_Communicator::start(Service * service)
{
  Communicator::start(service);
}

/*********************************************************
 * descriptor/endpoint accessor.
 * POST: RV == myDescriptor.
 *********************************************************/

inline int TLI_Communicator::getDescriptor() const
{
  return myDescriptor;
}

/**********************************************
 * stop(close) the socket.
 * POST: I have been stopped, and all of my
 *     dynamically-allocated memory has been reclaimed.
 **********************************************/

inline void TLI_Communicator::stop()
{
  Communicator::stop();
  delete myAddressInfo;
  std::t_close(getDescriptor());
}

/**************************************************
 * Destructor.
 * POST: If I was started, then I am stop.
 **************************************************/

inline TLI_Communicator::~TLI_Communicator()
{
  if (isOn())
    stop();
}


/******************************************************
 * SocketAddress accessor
 * POST: RV == myAddressInfo.
 ******************************************************/

inline SocketAddress * TLI_Communicator::addressInfo() const
{
  return myAddressInfo;
}



/***************************************************
 * Class className accessor.
 * POST: RV == TLI_Communicator.
 ***************************************************/

inline string TLI_Communicator::className() const
{
  return "TLI_Communicator";
}


#endif
