/* UDP_Server_Winsock_Communicator.h defines the Communicator abstraction
 *  for UDP server sockets under Winsock.
 *
 * Author: Joel Adams.
 * Date: November 1998
 * Version: 0.9
 *
 * Modified by: Ryan VandrBijl
 * Date:        May/June 2000
 */

#ifndef UDP_SERVER_Winsock_TRANSPORTER
#define UDP_SERVER_Winsock_TRANSPORTER

#include "Server_Winsock_Communicator.h"
#include "UDP_Service.h"
using namespace std;


class UDP_Server_Winsock_Communicator : public Server_Winsock_Communicator
{
 public:
  UDP_Server_Winsock_Communicator();

  UDP_Server_Winsock_Communicator(const string & serviceName);
  UDP_Server_Winsock_Communicator(const unsigned short port);
  UDP_Server_Winsock_Communicator(const string & serviceName, 
			const unsigned short port);
  virtual ~UDP_Server_Winsock_Communicator();
  UDP_Server_Winsock_Communicator(const UDP_Server_Winsock_Communicator & comm);

  void open(const string & serviceName);
  void open(const unsigned short port);
  void open(const string & serviceName, 
	    const unsigned short port);

  virtual void acceptConnection();    // first contact by a client
  string firstMessageFromClient();    // retrieve first contact msg
  virtual void closeConnection();     // solely for interface compliance
  virtual void forgetConnection() {};

  virtual string className() const;
  UDP_Server_Winsock_Communicator &
      operator=(const UDP_Server_Winsock_Communicator & comm);
  
  virtual int receive(string & msg);
  virtual int send   (const string & msg);

 protected:

 private:
  string myFirstMessageFromClient;
};

/*****************************************************************
 * Copy Constructor
 * PRE: comm is an object which inherits fromUDP_Server_Winsock_Communicator
 * POST: data members are equal to comm's data memebers.
 *****************************************************************/
inline UDP_Server_Winsock_Communicator::UDP_Server_Winsock_Communicator(const UDP_Server_Winsock_Communicator & comm)
   : Server_Winsock_Communicator(comm)
{
    this->myFirstMessageFromClient = comm.myFirstMessageFromClient;
}

/*****************************************************************
 * send - send a message to a client.
 * PRE: msg is a string containing the text to send.
 * POST: msg has been sent to the client's address 
 *****************************************************************/
inline int UDP_Server_Winsock_Communicator::send(const string & msg) {
   return ::sendto(getDescriptor(), msg.c_str(), msg.length(), 0,
		      client().addressInfoOldFormat(),
		      client().size());
}


/****************************************************
 * Default constructor
 * POST: Inherited data members have default values.
 ****************************************************/
inline UDP_Server_Winsock_Communicator::UDP_Server_Winsock_Communicator()
  : Server_Winsock_Communicator()
{}


/***************************************************
 * open with string-for-service.
 * PRE: service-name is the name of a valid UDP service.
 * POST: myServiceInfo points to a UDP_Service for serviceName.
 ***************************************************/
inline void UDP_Server_Winsock_Communicator::open(const string& serviceName)
{
  Server_Winsock_Communicator::open(new UDP_Service(serviceName));
}



/***************************************************
 * Construct with string-for-service.
 * PRE: service-name is the name of a valid UDP service.
 * POST: inherited data members have been initialized,
 *       myServiceInfo points to the Service for serviceName.
 ***************************************************/
inline UDP_Server_Winsock_Communicator::
UDP_Server_Winsock_Communicator(const string & serviceName)
  : Server_Winsock_Communicator()
{
  open(serviceName);
}


/***************************************************
 * open with port.
 * PRE: port is the port associated with a valid UDP service.
 * POST: myServiceInfo points to the UDP_Service for port.
 ***************************************************/
inline void UDP_Server_Winsock_Communicator::open(const unsigned short port)
{
  Server_Winsock_Communicator::open(new UDP_Service(port));
}


/***************************************************
 * Construct with port.
 * PRE: port is the port associated with a valid UDP service.
 * POST: inherited data members have been initialized,
 *       myServiceInfo points to a UDP_Service for port.
 ***************************************************/
inline UDP_Server_Winsock_Communicator::
UDP_Server_Winsock_Communicator(const unsigned short port)
  : Server_Winsock_Communicator()
{
  open(port);
}


/***************************************************
 * open with string-for-service, port
 *  (for defining one's own service).
 * PRE: serviceName is a user-defined service,
 *       port is the port associated with a valid UDP service,
 * POST: myServiceInfo points to a TCP_Service for serviceName, port.
 ***************************************************/
inline void UDP_Server_Winsock_Communicator::open(const string &serviceName,
					const unsigned short port)
{
  Server_Winsock_Communicator::open(new UDP_Service(serviceName, port));
}


/***************************************************
 * Construct with string-for-service, port
 *  (for defining one's own service).
 * PRE: serviceName is a user-defined service,
 *       port is the port associated with a valid UDP service,
 * POST: inherited data members have been initialized,
 *       myServiceInfo points to a TCP_Service for serviceName, port.
 ***************************************************/
inline UDP_Server_Winsock_Communicator::
UDP_Server_Winsock_Communicator(const string & serviceName, 
		      const unsigned short port)
  : Server_Winsock_Communicator()
{
  open(serviceName, port);
}



/***************************************************
 * Class className accessor
 * POST: RV == "UDP_Server_Winsock_Communicator".
 ***************************************************/
inline string UDP_Server_Winsock_Communicator::className() const
{
  return "UDP_Server_Winsock_Communicator";
}

/*****************************************************
 * acceptConnection first message from a client
 * (receive a message for UDP server transporters,
 *  which can be retrieved via firstMessageFromClient().)
 *****************************************************/
inline void UDP_Server_Winsock_Communicator::acceptConnection()
{
  if (receive(myFirstMessageFromClient) < 0) 
     warning("no message was received ...\n");
}

/*****************************************************
 * Retrieve first message sent by client
 * PRE: I have accepted a message from a client.
 * POST: RV == firstClientMessage.
 *****************************************************/
inline string UDP_Server_Winsock_Communicator::firstMessageFromClient()
{
  return myFirstMessageFromClient;
}


/*****************************************************
 * closeConnection from a connection
 * (do nothing for UDP Communicators,
 *  supplied solely for interface simplicity.)
 *****************************************************/
inline void UDP_Server_Winsock_Communicator::closeConnection()
{}


/***************************************************
 * Destructor (nothing to do)
 ***************************************************/

inline UDP_Server_Winsock_Communicator::~UDP_Server_Winsock_Communicator()
{}

#endif
