/* UDP_Service.h simplifies the interface for services using udp,
 *  but most of the functionality is inherited from Service.
 *
 * Author: Joel C. Adams.
 * Date: October, 1998
 * Version 0.8
 */

#ifndef UDP_SERVICE
#define UDP_SERVICE

#include "Service.h"                 // base class
#include "UDP_Protocol.h"            // UDP_Protocol abstraction
using namespace std;

class UDP_Service : public Service
{
 public:
  UDP_Service(const string & name);
  UDP_Service(const unsigned short port);
  UDP_Service(const string & name, const unsigned short port);
  UDP_Service(const unsigned short port, const string & name);
  virtual ~UDP_Service();

  virtual int BSD_Info() const;
  virtual char * TLI_Info() const;
  virtual void initProtocol();

  virtual string className() const;

 protected:
  UDP_Service();

 private:
};

/******************************************************
 * Default constructor (nothing to do)
 ******************************************************/

inline UDP_Service::UDP_Service()
{}

/******************************************************
 * Construct with TCP/IP service-name or port.
 * PRE: name is a valid TCP/IP service or a port number.
 * POST: isCopy == false and myProtocol == UDP &&
 *        myName == the name of the service (or 'user-defined') &&
 *        myPort == the (network byte order) port number.
 ********************************************************/

inline UDP_Service::UDP_Service(const string & name)
{
  initializeWith(name);   // from Service
}

/******************************************************
 * Construct with TCP/IP port.
 * PRE: port is the number of a TCP/IP (or unassigned) port.
 * POST: isCopy == false and myProtocol == UDP &&
 *        myName == the name of the service (or 'user-defined') &&
 *        myPort == the (network byte order) port number.
 ********************************************************/

inline UDP_Service::UDP_Service(const unsigned short port)
{
  initializeWith(port);  // from Service
}

/******************************************************
 * Construct with name and TCP/IP port.
 *  (for creating a new service).
 * PRE: name is the name of the new service,
 *      port is the number of an unassigned port.
 * POST: isCopy == false and myProtocol == UDP &&
 *        myName == the name of the service &&
 *        myPort == the (network byte order) port number.
 ********************************************************/

inline UDP_Service::UDP_Service(const string & name, 
				const unsigned short port)
{
  initializeWith(name, port);  // from Service
}

/******************************************************
 * Construct with name and TCP/IP port.
 *  (for creating a new service).
 * PRE: name is the name of the new service,
 *      port is the number of an unassigned port.
 * POST: isCopy == false and myProtocol == UDP &&
 *        myName == the name of the service &&
 *        myPort == the (network byte order) port number.
 ********************************************************/

inline UDP_Service::UDP_Service(const unsigned short port,
				const string & name)
{
  initializeWith(name, port);  // from Service
}

/********************************************************
 * Polymorphic protocol initialization method used by
 *  Service::initializeWith() to set the inherited
 *  myProtocolInfo member.
 * POST: myProtocolInfo points to a UDP_Protocol object.
 ********************************************************/

inline void UDP_Service::initProtocol()
{
  setProtocolInfo(new UDP_Protocol());
}

/*********************************************************
 * Extract the BSD information from this service's protocol
 * POST: RV == myProtocol->BSD_Info().
 *********************************************************/

inline int UDP_Service::BSD_Info() const
{
  return protocolInfo()->BSD_Info();
}

/*********************************************************
 * Extract the TLI information from this service's protocol
 * POST: RV == myProtocol->TLI_Info().
 *********************************************************/

inline char * UDP_Service::TLI_Info() const
{
  return protocolInfo()->TLI_Info();
}

/********************************************************
 * Polymorphic className() method used for diagnostics
 * POST: RV == "UDP_Service".
 ********************************************************/

inline string UDP_Service::className() const
{
  return "UDP_Service";
}


/*********************************************************
 * Destructor stub (nothing to do).
 *********************************************************/

inline UDP_Service::~UDP_Service()
{
}

#endif




