#include "TLI_Communicator.h"

/********************************************************
 * Allocate the endpoint, setting myDescriptor.
 * (polymorphic, called from Communicator::Open()).
 * PRE: all data members except myDescriptor are initialized.
 * POST: myDescriptor has been initialized.
 ********************************************************/
void TLI_Communicator::allocate()
{
  /* get the descriptor */
  myDescriptor = t_open(protocolInfo()->TLI_Info(),O_RDWR,0);

  if (myDescriptor < 0)
    fatal("TLI_Communicator::allocate(): call to t_open failed\n");
}



/******************************************************
 * Address mutator
 * PRE: address is a pointer to a SocketAddress object.
 * POST: myAddressInfo == address.
 ******************************************************/

void TLI_Communicator::setAddressInfo(SocketAddress * address)
{
  if (myAddressInfo != NULL)
    delete myAddressInfo;

  myAddressInfo = address;
}


/***************************************************
 * compare() utility function
 * (polymorphic, used by relational ops).
 * PRE: netObj is a TLI_Communicator object.
 * POST: RV == 0 if we are the same TLI_Communicator,
 *            +X if I am > than netObj, and
 *            -X if I am < netObj
 *            (using descriptor values).
 ***************************************************/

int TLI_Communicator::compare(const NetworkObject & netObj) const
{
  const TLI_Communicator * ptr = dynamic_cast<const TLI_Communicator*>(&netObj);
  if (ptr == NULL) fatal("TLI_Communicator::compare");

  return getDescriptor() - ptr->getDescriptor();
}

/***************************************************
 * print() utility.
 * PRE: out is an open ostream.
 * POST: my description has been inserted into out.
 ***************************************************/

void TLI_Communicator::print(ostream & out) const
{
  Communicator::print(out);
  if (myAddressInfo) {
      out << ", descriptor: " << getDescriptor()
	  << ", using " 
	  << (*myAddressInfo);
  } else {
      out << ", descriptor: " << getDescriptor()
	  << ", using " 
	  <<  "(no address)";
  }
}


/***************************************************
 * operator= - implement the assignment operator
 * PRE: comm is a class which inherits TLI_Communicator
 * POST: data members are now equal to comm's data memebers
 ***************************************************/
TLI_Communicator & TLI_Communicator::operator=(const TLI_Communicator & comm)
{
   Communicator::operator=(comm);
   this->myDescriptor = comm.myDescriptor;
   this->myAddressInfo = comm.myAddressInfo;
   return *this;
}

/***************************************************
 * Copy Constructor
 * PRE: comm is a class which inherits TLI_Communicator
 * POST: data members are now equal to comm's data memebers
 ***************************************************/
TLI_Communicator::TLI_Communicator(const TLI_Communicator & comm)
   : Communicator(comm)
{
   this->myDescriptor = comm.myDescriptor;
   *(this->myAddressInfo) = *(comm.myAddressInfo);
}



/**************************************************
 * Generate fatal error
 * PRE: format is a format-string, a la printf...
 * POST: format and trailing arguments have been
 *        displayed, and the program terminated.
 **************************************************/

void TLI_Communicator::fatal(const char * format, ...) const
{
  fprintf(stderr, "\n\n*** Error: ");
  va_list args;
  va_start(args, format);
  vfprintf(stderr, format, args);
  va_end(args);
  t_error("\nTLI Error Message");
  if (t_errno == TSYSERR)
      perror("System Error Message:");
  fprintf(stderr, "\n\n");
  exit(1);
}

/**************************************************
 * Generate (nonfatal) warning
 * PRE: format is a format-string, a la printf...
 * POST: format and trailing arguments have been
 *        displayed.
 **************************************************/

void TLI_Communicator::warning(const char * format, ...) const
{
  fprintf(stderr, "\n\n* warning: ");
  va_list args;
  va_start(args, format);
  vfprintf(stderr, format, args);
  va_end(args);
  t_error("\n");
  fprintf(stderr, "\n\n");
}
